package route

import (
	"github.com/gin-gonic/gin"
	"youngee_m_api/handler"
	"youngee_m_api/handler/operate"
	"youngee_m_api/middleware"
	"youngee_m_api/model/http_model"
)

func InitRoute(r *gin.Engine) {

	r.POST("/login", handler.WrapCodeLoginHandler)
	r.GET("/getLoginUser", handler.WrapGetLoginUserHandler)
	r.POST("/userInfo", handler.WrapGetUserInfoHandler)
	r.GET("/test/ping", func(c *gin.Context) {
		resp := http_model.CommonResponse{
			Status:  0,
			Message: "",
			Data:    "ping",
		}
		c.JSON(200, resp)
	})
	m := r.Group("/youngee/m")
	{
		m.Use(middleware.LoginAuthMiddleware)
		m.POST("/test", func(c *gin.Context) {
			resp := http_model.CommonResponse{
				Status:  0,
				Message: "",
				Data:    "ping",
			}
			c.JSON(200, resp)
		})
		m.POST("/product/list", handler.WrapFullProjectListHandler)
		m.POST("/project/show", handler.WrapShowProjectHandler)
		m.POST("/project/handle", handler.WrapProjectHandleHandler)
		m.POST("/product/findall", handler.WrapFindAllProductHandler)
		m.POST("/project/create", handler.WrapCreateProjectHandler)
		m.POST("/product/create", handler.WrapCreateProductHandler)
		m.POST("/product/find", handler.WrapFindProductHandler)
		m.POST("/pay/paysum", handler.WrapPaySumHandler)
		m.POST("/project/update", handler.WrapUpdateProjectHandler)
		m.POST("/project/approve", handler.WrapApproveProjectHandler)
		m.POST("/project/all", handler.WrapGetAllProjectHandler)
		m.POST("/project/taskList", handler.WrapProjectTaskListHandler)
	}
	u := r.Group("/youngee/m/user")
	{
		u.Use(middleware.LoginAuthMiddleware)
		u.POST("/getUserList", handler.WrapGetUserListHandler)
		u.POST("/updateUserInfo", handler.WrapUpdateUserInfoHandler)
		u.POST("/createUser", handler.WrapCreateUserHandler)
		u.POST("/disabledUser", handler.WrapDisabledUserHandler)
		u.POST("/enterpriseUser", handler.WrapEnterpriseUserHandler)
		u.POST("/creatorList", handler.WrapCreatorListHandler)
		u.POST("/platformAccInfo", handler.WrapPlatformAccInfoHandler)
		u.POST("/talentInfo", handler.WrapTalentInfoHandler)
		u.POST("/accountInfo", handler.WrapAccountInfoHandler)
		u.POST("/deleteAccount", handler.WrapDeleteAccountHandler)
		u.POST("/getTaskRecord", handler.WrapGetTaskRecordHandler)
	}
	o := r.Group("/youngee/m/operate")
	{
		o.Use(middleware.LoginAuthMiddleware)
		o.POST("/addPricing", operate.WrapAddPricingHandler)
		o.POST("/searchPricing", operate.WrapSearchPricingHandler)
		o.POST("/modifyPricing", operate.WrapModifyPricingHandler)
		o.POST("/signInOffline", operate.WrapSignInOfflineHandler)
		o.POST("/signInVirtual", operate.WrapSignInVirtualHandler)
		o.POST("/autoReview", operate.WrapAutoReviewHandler)
		o.POST("/autoReviewUnlimited", operate.WrapAutoReviewUnlimitedHandler)
		o.POST("/postReview", operate.WrapPostReviewHandler)
		o.POST("/caseClose", operate.WrapCaseCloseHandler)
		o.POST("/invalid", operate.WrapInvalidHandler)
		o.POST("/draftDefaultInPic", operate.WrapDraftDefaultInPicHandler)
		o.POST("/draftDefaultInMv", operate.WrapDraftDefaultInMvHandler)
		o.POST("/scriptDefault", operate.WrapScriptDefaultHandler)
		o.POST("/linkBreach", operate.WrapLinkBreachHandler)
		o.POST("/caseCloseDefault", operate.WrapCaseCloseDefaultHandler)
		o.GET("/countNumOfDefaults", operate.WrapCountNumOfDefaultsHandler)
		o.POST("/breachPending", operate.WrapBreachPendingHandler)
		o.POST("/contractBreach", operate.WrapContractBreachHandler)
		o.POST("/getSketchInfoByTaskId", operate.WrapGetSketchInfoByTaskIdHandler)
		o.POST("/breachHandled", operate.WrapBreachHandledHandler)
		o.POST("/sketchReplaceNotUpload", operate.WrapSketchReplaceNotUploadHandler)
		o.POST("/sketchReplaceTimeOut", operate.WrapSketchReplaceTimeOutHandler)
		o.POST("/sketchOtherNotUpload", operate.WrapSketchOtherNotUploadHandler)
		o.POST("/sketchOtherTimeOut", operate.WrapSketchOtherTimeOutHandler)
		o.POST("/scriptReplaceNotUpload", operate.WrapScriptReplaceNotUploadHandler)
		o.POST("/scriptReplaceTimeOut", operate.WrapScriptReplaceTimeOutHandler)
		o.POST("/scriptOtherNotUpload", operate.WrapScriptOtherNotUploadHandler)
		o.POST("/scriptOtherTimeOut", operate.WrapScriptOtherTimeOutHandler)
		o.POST("/linkReplaceNotUpload", operate.WrapLinkReplaceNotUploadHandler)
		o.POST("/linkReplaceTimeOut", operate.WrapLinkReplaceTimeOutHandler)
		o.POST("/linkOtherNotUpload", operate.WrapLinkOtherNotUploadHandler)
		o.POST("/linkOtherTimeOut", operate.WrapLinkOtherTimeOutHandler)
		o.POST("/dataReplaceNotUpload", operate.WrapDataReplaceNotUploadHandler)
		o.POST("/dataReplaceTimeOut", operate.WrapDataReplaceTimeOutHandler)
		o.POST("/dataOtherNotUpload", operate.WrapDataOtherNotUploadHandler)
		o.POST("/dataOtherTimeOut", operate.WrapDataOtherTimeOutHandler)
	}
	f := r.Group("/youngee/m/finance")
	{
		f.Use(middleware.LoginAuthMiddleware)
		f.POST("/withdrawalRecords", handler.WrapWithdrawalRecordsHandler)
		f.POST("/getWithdrawalRecord", handler.WrapGetWithdrawalRecordHandler)
	}
}
